﻿using EmailClientWPF.Commands;
using EmailClientWPF.Data;
using EmailClientWPF.Helpers;
using System;
using System.Collections.Generic;
using System.Windows.Input;

namespace EmailClientWPF.ViewModels
{
    public class AppViewModel : ViewModelBase
    {
        private ICommand changeViewModelCommand;
        private IPageViewModel currentViewModel;
        private bool loggedInUser = false;
        private ICommand logoutCommand;

        public string Username { get; set; }


        public IPageViewModel CurrentViewModel
        {
            get
            {
                return this.currentViewModel;
            }
            set
            {
                this.currentViewModel = value;
                this.OnPropertyChanged("CurrentViewModel");
            }
        }

        public bool LoggedInUser
        {
            get
            {
                return this.loggedInUser;
            }
            set
            {
                this.loggedInUser = value;
                this.OnPropertyChanged("LoggedInUser");
            }
        }

        //Add new view models as properties here and then add to the ViewModels.Add in constructor
        public LoginRegisterFormViewModel LoginRegisterVM { get; set; }
        public List<IPageViewModel> ViewModels { get; set; }
        
        //Log out
        public ICommand Logout
        {
            get
            {
                if (this.logoutCommand == null)
                {
                    this.logoutCommand = new RelayCommand(this.HandleLogoutCommand);
                }
                return this.logoutCommand;
            }
        }

        //When we logout user change the current view to LogginRegisterView
        private void HandleLogoutCommand(object obj)
        {
            bool isUserLoggedOut = DataPersister.LogoutUser();
            if (isUserLoggedOut)
            {
                this.Username = "";
                this.LoggedInUser = false;
                //this.CurrentViewModel = this.LoginRegisterVM;
                this.HandleChangeViewModelCommand(this.LoginRegisterVM);
            }
        }

        //Change the Current ViewModel
        public ICommand ChangeViewModel
        {
            get
            {
                if (this.changeViewModelCommand == null)
                {
                    this.changeViewModelCommand =
                        new RelayCommand(this.HandleChangeViewModelCommand);
                }
                return this.changeViewModelCommand;
            }
        }

        public event EventHandler<ViewChangedArgs> ViewChanged;

        private void HandleChangeViewModelCommand(object parameter)
        {
            var pageModel = parameter as IPageViewModel;
            var newCurrentViewModel = pageModel;
            this.CurrentViewModel = newCurrentViewModel;
            this.RaiseViewChanged(pageModel.Name);
        }

        //Add instance the view models and the first view = LoginRegisterView
        public AppViewModel()
        {
            this.ViewModels = new List<IPageViewModel>();
            var loginVM = new LoginRegisterFormViewModel();
            var newMessage = new NewMessageViewModel();
            var inboxVM = new InboxViewModel();
            var sentVM = new SentViewModel();
            var trashVM = new TrashViewModel();

            this.ViewModels.Add(newMessage);
            this.ViewModels.Add(inboxVM);
            this.ViewModels.Add(sentVM);
            this.ViewModels.Add(trashVM);

            this.ViewChanged += inboxVM.ViewChanged;
            this.ViewChanged += sentVM.ViewChanged;
            this.ViewChanged += trashVM.ViewChanged;
            newMessage.MessageSent += this.MessageSent;
            loginVM.LoginSuccess += this.LoginSuccessful;
            this.LoginRegisterVM = loginVM;
            this.CurrentViewModel = this.LoginRegisterVM;
        }

        //When user loggs in succesful change the view to see his EmailBox with all the views - Inbox, SendedMessages, Trash and LogOutButton
        
        public void LoginSuccessful(object sender, LoginSuccessArgs e)
        {
            this.Username = e.Username;
            this.LoggedInUser = true;
            this.HandleChangeViewModelCommand(this.ViewModels[1]);
        }

        public void MessageSent(object sender, MessageSentArgs e)
        {
            this.HandleChangeViewModelCommand(this.ViewModels[1]);
        }

        private void RaiseViewChanged(string viewName)
        {
            var viewChangedEvent = ViewChanged;
            if (viewChangedEvent != null)
            {
                viewChangedEvent(this, new ViewChangedArgs(viewName));
            }
        }
    }
}
